VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Packet"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" _
    (ByRef Destination As Any, ByRef Source As Any, ByVal numbytes As Long)

Private Enum PacketType
    pIncoming
    pOutgoing
    pNull
End Enum

Public Enum PacketFormat
    pBnet
    pBNLS
    pNone
End Enum

Private RawData As String
Private Mode As PacketType
Public Position As Long

Public Sub Reset()
    Mode = pNull
    RawData = ""
    Position = 1
End Sub

Public Function SetData(Data As String)
    RawData = Data
    Mode = pIncoming
End Function

Public Function GetData() As String
    GetData = RawData
    Reset
End Function

Public Sub Skip(Bytes As Long)
    Position = Position + Bytes
End Sub

Public Function GetString() As String
    Dim NTPos&, strTemp$
    GetString = ""
    NTPos = InStr(Position, RawData, Chr$(0))
    If NTPos = 0 Then
        Err.Raise 20000, , "Trying to read past end of packet."
        Exit Function
    End If
    GetString = Mid$(RawData, Position, NTPos - Position)
    Position = NTPos + 1
End Function

Public Function GetFixedString(Length As Long) As String
    Dim rVal&
    If LenData() - Position + 1 >= Length Then
        GetFixedString = Mid$(RawData, Position, Length)
        Position = Position + Length
    Else
        Err.Raise 20000, , "Trying to read past end of packet."
        Exit Function
    End If
End Function

Public Function GetBoolean() As Boolean
    Dim rVal&
    If LenData() - Position + 1 >= 4 Then
        CopyMemory rVal, ByVal Mid$(RawData, Position, 4), 4
        Position = Position + 4
    Else
        Err.Raise 20000, , "Trying to read past end of packet."
        Exit Function
    End If
    If rVal = 0 Then
        GetBoolean = False
    Else
        GetBoolean = True
    End If
End Function

Public Function GetDWORD() As Long
    Dim rVal&
    If LenData() - Position + 1 >= 4 Then
        CopyMemory rVal, ByVal Mid$(RawData, Position, 4), 4
        Position = Position + 4
        GetDWORD = rVal
        'BotEvent "", evDbg, Position & ":" & StrToHex(Mid$(RawData, Position))
    Else
        Err.Raise 20000, , "Trying to read past end of packet."
        Exit Function
    End If
End Function

Public Function GetWORD() As Integer
    Dim rVal%
    If LenData() - Position + 1 >= 2 Then
        CopyMemory rVal, ByVal Mid$(RawData, Position, 2), 2
        Position = Position + 2
        GetWORD = rVal
    Else
        Err.Raise 20000, , "Trying to read past end of packet."
        Exit Function
    End If
End Function

Public Function GetByte() As Byte
    Dim rVal As Byte
    rVal = 0
    If LenData() - Position + 1 >= 1 Then
        CopyMemory rVal, ByVal Mid$(RawData, Position, 1), 1
        Position = Position + 1
        GetByte = rVal
    Else
        GetByte = 0
    End If
End Function

Public Function GetStringArray(ByVal Length As Long) As String()
    If Length = 0 Then Exit Function
    Dim OS() As String, i&
    ReDim OS(Length) As String
    For i = 0 To Length - 1
        OS(i) = GetString()
    Next i
    GetStringArray = OS
End Function

Public Function LenData() As Long
    LenData = Len(RawData)
End Function

Public Function Send(Socket As Long, Optional PacketID As Byte, Optional ByVal Format As PacketFormat = pBnet)
    If Mode = pIncoming Then Exit Function
    Select Case Format
        Case pBnet
            SendData Socket, Chr$(&HFF) & MakeByte(PacketID) & _
                MakeWORD(Len(RawData) + 4) & RawData
        Case pBNLS
            SendData Socket, MakeWORD(Len(RawData) + 3) & MakeByte(PacketID) & RawData
        Case Else
            SendData Socket, RawData
    End Select
End Function

Public Sub InsertString(Data As String)
    If Mode = pIncoming Then Exit Sub
    Mode = pOutgoing
    RawData = RawData & Data & Chr$(0)
End Sub

Public Sub InsertStringList(Data() As String)
On Error Resume Next
    If Mode = pIncoming Then Exit Sub
    Mode = pOutgoing
    Dim i As Integer
    For i = LBound(Data) To UBound(Data) Step 1
        RawData = RawData & Data(i) & Chr$(0)
    Next i
    RawData = RawData & Chr$(0) 'extra null byte to end stringlist
End Sub

Public Sub InsertStringArray(Data() As String)
On Error Resume Next
    If Mode = pIncoming Then Exit Sub
    Mode = pOutgoing
    Dim i As Integer
    For i = LBound(Data) To UBound(Data) Step 1
        RawData = RawData & Data(i) & Chr$(0)
    Next i
    'no extra null byte in string arrays
End Sub

Public Sub InsertNTString(Data As String)
    InsertString Data
End Sub

Public Sub InsertNonNTString(Data As String)
    If Mode = pIncoming Then Exit Sub
    Mode = pOutgoing
    RawData = RawData & Data
End Sub

Public Sub InsertByte(Data As Byte)
    If Mode = pIncoming Then Exit Sub
    Mode = pOutgoing
    
    RawData = RawData & MakeByte(Data)
End Sub

Public Sub InsertWORD(Data As Integer)
    If Mode = pIncoming Then Exit Sub
    Mode = pOutgoing
    
    RawData = RawData & MakeWORD(Data)
End Sub

Public Sub InsertDWORD(Data As Long)
    If Mode = pIncoming Then Exit Sub
    Mode = pOutgoing
    
    RawData = RawData & MakeDWORD(Data)
End Sub

Public Sub InsertDWORDArray(Data() As Long)
    If Mode = pIncoming Then Exit Sub
    Mode = pOutgoing
    
    Dim i As Integer
    For i = LBound(Data) To UBound(Data) Step 1
        RawData = RawData & MakeDWORD(Data(i))
    Next i
End Sub

Private Function MakeByte(Data As Byte) As String
    Dim Result As String * 1
    CopyMemory ByVal Result, Data, 1
    MakeByte = Result
End Function

Private Function MakeWORD(Data As Integer) As String
    Dim Result As String * 2
    CopyMemory ByVal Result, Data, 2
    MakeWORD = Result
End Function

Private Function MakeDWORD(Data As Long) As String
    Dim Result As String * 4
    CopyMemory ByVal Result, Data, 4
    MakeDWORD = Result
End Function

Private Sub Class_Initialize()
    Position = 1
    Mode = pNull
End Sub
